#ifndef _fbas__defines_h_
#define _fbas__defines_h_

typedef short fbas_int_t;
#define fbas_chr yyctx->chr
#define fbas_val yyctx->val
#define fbas_lval yyctx->lval
#define fbas_stack yyctx->stack
#define fbas_debug yyctx->debug
#define fbas_nerrs yyctx->nerrs
#define fbas_errflag yyctx->errflag
#define fbas_state yyctx->state
#define fbas_yyn yyctx->yyn
#define fbas_yym yyctx->yym
#define fbas_jump yyctx->jump
#line 2 "fbas.y"
/*
libfawk - A function-only AWK dialect (BASIC compiler)

Copyright (c) 2020 Tibor 'Igor2' Palinkas. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

  * 1. Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
  * 3. Neither the name of the copyright holder nor the names of its
    contributors may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Project page: http://repo.hu/projects/libfawk
Source code: svn://repo.hu/libfawk/trunk
Contact the author: http://igor2.repo.hu/contact.html
*/

#define fbas_ctx_t fawk_ctx_t

#include "config.h"
#include <stdlib.h>
#include <stdio.h>
#include <genht/hash.h>
#include "libfawk.h"
#include "parser.h"
#include "fbas.y.tab.h"
#include "parser_fbas.h"
#include "symtab.h"
#include "compile.h"
#include "execute.h"

#define FAWK_PUSH_IP() fawk_push_num(ctx, ctx->code.used)
#define FAWK_CURR_IP() ctx->code.used

#include "label_inlines.h"
#include "mainprog_inlines.h"

#line 57 "fbas.y"
typedef union {
	char *str;
	fawk_num_t num;
} fbas_tokunion_t;
typedef fbas_tokunion_t fbas_STYPE;


#define BT_NEQ 257
#define BT_GTEQ 258
#define BT_LTEQ 259
#define BT_NIL 260
#define BT_IF 261
#define BT_THEN 262
#define BT_ELSE 263
#define BT_FOR 264
#define BT_TO 265
#define BT_STEP 266
#define BT_NEXT 267
#define BT_DO 268
#define BT_WHILE 269
#define BT_UNTIL 270
#define BT_LOOP 271
#define BT_GOTO 272
#define BT_DEF 273
#define BT_END 274
#define BT_LET 275
#define BT_CURRFUNC 276
#define BT_ID 277
#define BT_STRING 278
#define BT_NUM 279
#define BT_OR 280
#define BT_AND 281
#define BT_IN 282
#define BT_MOD 283
#define BT_NOT 284
#define fbas_ERRCODE 256

#ifndef fbas_INITSTACKSIZE
#define fbas_INITSTACKSIZE 200
#endif

typedef struct {
	unsigned stacksize;
	fbas_int_t *s_base;
	fbas_int_t *s_mark;
	fbas_int_t *s_last;
	fbas_STYPE *l_base;
	fbas_STYPE *l_mark;
#if fbas_DEBUG
	int debug;
#endif
} fbas_STACKDATA;

typedef struct {
	int errflag;
	int chr;
	fbas_STYPE val;
	fbas_STYPE lval;
	int nerrs;
	int yym, yyn, state;
	int jump;
	int stack_max_depth;
	int debug;

	/* variables for the parser stack */
	fbas_STACKDATA stack;
} fbas_yyctx_t;

typedef enum { fbas_RES_NEXT, fbas_RES_DONE, fbas_RES_ABORT } fbas_res_t;

FAWK_API int fbas_parse_init(fbas_yyctx_t *yyctx);
FAWK_API fbas_res_t fbas_parse(fbas_yyctx_t *yyctx, fbas_ctx_t *ctx, int tok, fbas_STYPE *lval);
FAWK_API void fbas_error(fbas_ctx_t *ctx, fbas_STYPE tok, const char *msg);


#endif /* _fbas__defines_h_ */
